import copy
from abc import ABC, abstractmethod


class FootballPlayer(ABC):

    def __init__(self, name, salary, performance):
        self.name = name
        self.salary = salary
        self.performance = float(performance)

    @abstractmethod
    def play(self, num):
        pass

    def __repr__(self):
        return f"Name: {self.name}\n" \
               f"Salary: {self.salary}M $\n" \
               f"Performance: {self.performance}"


class OffensePlayer(FootballPlayer):

    def __init__(self, name, salary, performance):
        super().__init__(name, salary, performance)  # FootballPlayer.__init__(self, name, salary, performance)
        self.total_yards = 0

    def play(self, num):
        self.total_yards += num

    def get_total_yards(self):
        return self.get_total_yards

    def __repr__(self):
        res = super().__repr__()  # FootballPlayer.__repr__(self)
        return f'{res}\nTotal Yards: {self.total_yards}'


class DefensePlayer(FootballPlayer):

    def __init__(self, name, salary, performance):
        super().__init__(name, salary, performance)
        self.total_tackles = 0

    def play(self, num=1):
        self.total_tackles += num

    def __repr__(self):
        res = super().__repr__()
        return f'{res}\nTotal Tackles: {self.total_tackles}'


class FootballTeam:
    """
    This class represents a Football Team
    """

    def __init__(self, players):
        """
        Constructor.
        :param players: list - list of Football Players objects
        """
        self.__players = players

    def get_team(self):
        """
        method that return a copy of the players list
        """
        # return self.__players
        return copy.deepcopy(self.__players)

    def get_name(self, i):
        return self.__players[i].name


p1 = DefensePlayer('Janis Griffin', 15, 6.2)
p2 = OffensePlayer('John Smith', 22, 7)
team = FootballTeam([p1, p2])

for player in team.get_team():
    player.play(1)
